/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef POINT_HH
#define POINT_HH

#include "config.h"
#include <vector>

// For gcc, at least, __SSE__ is defined to indicate whether SSE is enabled
#ifdef __SSE__
#include "Align.hh"
#include <xmmintrin.h>
#endif

/* At long last, the (hopefully) portable successor to Optimized.h */

namespace SharpConstruct
{
	class Point3D
	{
	public:
		inline Point3D();
		inline Point3D( const float, const float, const float );

		inline float& X();
		inline float& Y();
		inline float& Z();

		inline float X() const;
		inline float Y() const;
		inline float Z() const;

		inline void operator+=( const Point3D& );
		inline void operator-=( const Point3D& );
		inline void operator*=( const Point3D& );
		inline void operator/=( const Point3D& );
		inline void operator+=( const float );
		inline void operator-=( const float );
		inline void operator*=( const float );
		inline void operator/=( const float );

		inline Point3D operator+( const Point3D& ) const;
		inline Point3D operator-( const Point3D& ) const;
		inline Point3D operator*( const Point3D& ) const;
		inline Point3D operator/( const Point3D& ) const;
		inline Point3D operator+( const float ) const;
		inline Point3D operator-( const float ) const;
		inline Point3D operator*( const float ) const;
		inline Point3D operator/( const float ) const;
	private:
#ifdef __SSE__
		inline float& component_( const unsigned );
		inline float component_( const unsigned ) const;
#ifndef _MSC_VER
		__m128 data_;
#else
                float data_[4];
#endif
	public:
		inline Point3D( const __m128& );

#ifndef _MSC_VER
		inline const __m128& Data() const;
#else
		// Defined slightly different, to avoid warning C4172 on MSVC
		inline const __m128 Data() const;
#endif
#else
		float x_, y_, z_;
#endif
	};

	inline Point3D Cross( const Point3D&, const Point3D& );
	inline float Distance( const Point3D&, const Point3D& );
	inline float Dot( const Point3D&, const Point3D& );
	inline float HorizontalAdd( const Point3D& );
	inline Point3D Midpoint( const Point3D&, const Point3D& );
	inline void Normalize( Point3D& );
	inline Point3D Origin();
	inline Point3D PlaneNormal( const Point3D&, const Point3D&, const Point3D& );
	inline Point3D PlaneUnormal( const Point3D&, const Point3D&, const Point3D& );
		
	typedef Point3D Normal3D;

#ifdef __SSE__
	typedef std::vector< Point3D, Align< Point3D > > Point3DVector;
#else
	typedef std::vector< Point3D > Point3DVector;
#endif

	void Normalize( Point3DVector& );
};

// Inline definitions for Point3D are in these files
#ifdef __SSE__
#ifndef _MSC_VER
#include "PointImplSSE.hh"
#else
#include "PointImplSSE_MSVC.hh"
#endif
#else
#include "PointImpl.hh"
#endif

#endif
