/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef POLYGON3D_H
#define POLYGON3D_H

#include <vector>

namespace SharpConstruct
{
	class Edge
	{
	public:
		Edge( const unsigned v1, const unsigned v2 )
		: HasMid( false ), Mid( 0 )
		{
			V[ 0 ] = v1;
			V[ 1 ] = v2;
		}

		unsigned V[ 2 ];
		
		bool HasMid;
		unsigned Mid;
	};

	class TriangleEdges
	{
	public:
		static const unsigned Components = 3;

		const unsigned& operator[]( const unsigned i ) const
		{
			return edges_[ i ];
		}
		unsigned& operator[]( const unsigned i )
		{
			return edges_[ i ];
		}
	private:
		unsigned edges_[ 3 ];
	};

	class QuadEdges
	{
	public:
		/*QuadEdges()
		{
			for( unsigned i = 0; i < 4; ++i )
				E[ i ] = 0;
		}*/
		static const unsigned Components = 4;

		const unsigned& operator[]( const unsigned i ) const
		{
			return E[ i ];
		}
		unsigned& operator[]( const unsigned i )
		{
			return E[ i ];
		}

		unsigned E[ 4 ];
	};

	class EdgeData
	{
	public:
		std::vector< Edge > Edges;
		std::vector< TriangleEdges > Triangles;
		std::vector< QuadEdges > Quads;
	};

	/*template< typename T > bool Complete( const T& t )
	{
		for( unsigned i = 0; i < T::Components; ++i )
			if( t.E[ i ] == 0 )
				return false;
		return true;
	}*/

	template< typename T > unsigned SubCount( const T& t, const std::vector< Edge >& edges )
	{
		unsigned c( 0 );
		for( unsigned i = 0; i < T::Components; ++i )
			if( edges[ t[ i ] ].HasMid )
				++c;
		return c;
	}


	class Triangle
	{
	public:
		Triangle()
		{}
		Triangle( unsigned i1, unsigned i2, unsigned i3 );
			
		Triangle& ReverseVertices();
	
		unsigned V[ 3 ];
	};
	class Quad
	{
	public:
		Quad()
		{}
		Quad( unsigned i1, unsigned i2, unsigned i3, unsigned i4 );
			
		Quad& ReverseVertices();
	
		unsigned V[ 4 ];
	};
	
	class PolygonUsers
	{
	public:
		unsigned Total() const
		{
			return Triangles.size() + Quads.size();
		}

		std::vector< unsigned > Triangles;
		std::vector< unsigned > Quads;
	};

	// Mostly a temp structure
	class Polygon3D
	{
	public:
		Polygon3D();
		~Polygon3D();
		
		std::vector< unsigned > VertexIndices;
		std::vector< unsigned > UVIndices;
	};
}

#endif // POLYGON3D_H
