/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Log.hh"
#include "MainWindow.hh"
#include <gtkmm/main.h>
#include <iostream>

#ifdef _MSC_VER
#include "win32.h"
#include "usecrashhandler.inc"
#endif

int main( int argc, char** argv )
{
	using namespace SharpConstruct;

	Log::Instance().Add( "Initializing log file" );

#ifdef _MSC_VER
	initCrashHandler("SharpConstruct","christl@users.sourceforge.net");
	Log::Instance().Add( "Done initializing crash handler" );
#endif
	bool argversion( false );

	Glib::OptionContext options( "[file]" );
	Glib::OptionGroup appgroup( "Appoptions", "arg1", "arg2" );
	Glib::OptionEntry version;
	version.set_long_name( "version" );
	version.set_short_name( 'v' );
	version.set_description( "Print program version information and exit" );
	appgroup.add_entry( version, argversion );
	options.set_main_group( appgroup );

	try
	{
		options.parse( argc, argv );
	}
	catch( Glib::OptionError )
	{
		std::cout << "Bad arguments, ignoring" << std::endl;
	}

	if( argversion )
	{
		std::cout << "SharpConstruct " << SharpConstruct::GetVersion() <<
			"\nCopyright (c) 2006 Nicholas Bishop.\n"
			"SharpConstruct comes with NO WARRANTY, to the extent permitted by law.\n"
			"You may redistribute copies of SharpConstruct under the terms of the\n"
			"GNU General Public License. For more information about these matters,\n"
			"see the file named COPYING.\n" << std::endl;
		return 0;
	}

	Log::Instance().Add( "Done parsing arguments" );

	Gtk::Main kit( argc, argv );
	Gtk::GL::init( argc, argv );
	Gdk::GL::init( argc, argv );
	Log::Instance().Add( "Initialized gtkmm and gtkglextmm libraries" );

	GInterface::MainWindow mainwindow;
	Log::Instance().Add( "MainWindow constructed" );
	mainwindow.Run( kit );

#ifdef _MSC_VER
	doneCrashHandler();
#endif
}
