/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MeshHistory.hh"
#include "Mirror.hh"

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

using SharpConstruct::Mirror;

Mirror::Mirror()
{
	mirrors_[ X ] = mirrors_[ Y ] = mirrors_[ Z ] = false;
}

void Mirror::FlatDraw( const Gdk::Point&, const Gdk::Point& )
{}

void Mirror::ModifyModel()
{
	const int flip( mirrors_[ X ] + mirrors_[ Y ] + mirrors_[ Z ] );
	if( flip == 1 || flip == 3 )
		glFrontFace( GL_CCW ); // This is reverted in Editor.cc
	glScalef( !mirrors_[ X ] * 2 - 1,
		  !mirrors_[ Y ] * 2 - 1,
		  !mirrors_[ Z ] * 2 - 1);
}

void Mirror::MouseUp( const Gdk::Point& )
{
	if( selected_ != NONE )
		mirrors_[ over_ ] = !mirrors_[ over_ ];
	selected_ = NONE;
}


void Mirror::Apply()
{
	MeshHistory::Instance().AddMesh( true, false, false );
	MeshHistory::Instance().GetCurrentMesh().Mirror( mirrors_[ X ],
							 mirrors_[ Y ],
							 mirrors_[ Z ] );
}

void Mirror::draw_handle_( const Axis a, const float x, const float y, const float z )
{
	const float delta( 10 );
	const float back( delta * -2 );

	GLUquadric* quadric = gluNewQuadric();

	glPushMatrix();
	glRotatef( 90, x, y, z );
	glTranslatef( delta, 0, 0 );
	gluSphere( quadric, 1.5, 10, 10 );
	glTranslatef( back, 0, 0 );
	gluSphere( quadric, 1.5, 10, 10 );
	glPopMatrix();

	glPushMatrix();
	glRotatef( 90, z, x, y );
	glTranslatef( 0, 0, -delta );
	gluCylinder( quadric, 0.5, 0.5, delta * 2, 10, 1 );
	glPopMatrix();

	gluDeleteQuadric( quadric );
}
