/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#if 0


#include "Brush.h"
#include "MeshHistory.h"
#include "Polygon3D.h"
#include "SubdivideBrush.hh"

using SharpConstruct::SubdivideBrush;
using SharpConstruct::SculptPhase;
using SharpConstruct::Mesh;
using SharpConstruct::SculptStart;
using SharpConstruct::SculptContinue;
using SharpConstruct::EditData;

unsigned SubdivideBrush::vsize_;
unsigned SubdivideBrush::tsize_;
unsigned SubdivideBrush::qsize_;

SubdivideBrush::SubdivideBrush( SculptPhase, Mesh& mesh )
{
	mesh.ClearModifiedVertexIndices();
}

void SubdivideBrush::operator()( SculptStart, EditData e )
{
	Mesh& mesh( MeshHistory::Instance().GetCurrentMesh() );
	vsize_ = mesh.VertexLocations().size();
	tsize_ = mesh.Triangles().size();
	qsize_ = mesh.Quads().size();
}

void SubdivideBrush::operator()( SculptContinue, EditData e )
{
	using Optimized::Point3D;

	Mesh& mesh( MeshHistory::Instance().GetCurrentMesh() );
	const MeshHistory::PartialMesh::PartialPolygonData& ppd( *MeshHistory::Instance().GetPreviousComposite().Polygons() );
	const EdgeData& oldedges( ppd.Edges() );

	const std::vector< Edge >& edges( oldedges.Edges );
	const std::vector< TriangleEdges >& tri_edges( oldedges.Triangles );
	const std::vector< QuadEdges >& quad_edges( oldedges.Quads );

	/*mesh.VertexLocations().resize( vsize_ );
	mesh.Triangles().resize( tsize_ );
	mesh.Quads().resize( qsize_ );
	mesh.VertexUsers().resize( vsize_ );
	mesh.MakeSane();*/
	mesh.VertexLocations().resize( vsize_ );
	mesh.Triangles() = ppd.Triangles();
	mesh.Quads() = ppd.Quads();
	mesh.VertexUsers() = ppd.VertexUsers();
	//mesh.Edges().Triangles = ppd.Edges().Triangles;
	//mesh.Edges().Quads = ppd.Edges().Quads;

	mesh.FindActiveVertices( active_data_, e );
	mesh.RecalculateDamaged();

	//std::cout << mesh.DamagedQuads().size() << std::endl;

	const unsigned tri_size( mesh.DamagedTriangles().size() );
	for( unsigned i = 0; i < tri_size; ++i )
	{
		const unsigned poly( mesh.DamagedTriangles()[ i ] );
		for( unsigned j = 0; j < 2; ++j )
		{
			Edge& edge( mesh.Edges().Edges[ tri_edges[ poly ][ j ] ] );
			if( !edge.HasMid )
			{
				edge.Mid = mesh.VertexLocations().size();
				mesh.VertexLocations().resize( mesh.VertexLocations().size() + 1 );
				mesh.VertexLocations()[ edge.Mid ].Midpoint( mesh.VertexLocations()[ edge.V[ 0 ] ], mesh.VertexLocations()[ edge.V[ 1 ] ] );
				edge.HasMid = true;
			}
		}
	}

	const unsigned quad_size( mesh.DamagedQuads().size() );
	for( unsigned i = 0; i < quad_size; ++i )
	{
		const unsigned poly( mesh.DamagedQuads()[ i ] );
		for( unsigned j = 0; j < 3; ++j )
		{
			Edge& edge( mesh.Edges().Edges[ quad_edges[ poly ].E[ j ] ] );
			if( !edge.HasMid )
			{
				edge.Mid = mesh.VertexLocations().size();
				mesh.VertexLocations().resize( mesh.VertexLocations().size() + 1 );
				mesh.VertexLocations()[ edge.Mid ].Midpoint( mesh.VertexLocations()[ edge.V[ 0 ] ], mesh.VertexLocations()[ edge.V[ 1 ] ] );
				edge.HasMid = true;
			}
		}
	}

	for( unsigned i = 0; i < tri_edges.size(); ++i )
	{
		const Triangle poly( mesh.Triangles()[ i ] );
		const TriangleEdges& es( tri_edges[ i ] );
		const unsigned count( SubCount( es, edges ) );

		if( count == 3 )
		{
			/*
			     /\
			    /__\
			   /\  /\
			  /__\/__\

7			*/

			mesh.Triangles().push_back( Triangle( poly.V[ 0 ], edges[ es[ 0 ] ].Mid, edges[ es[ 2 ] ].Mid ) );
			mesh.Triangles().push_back( Triangle( poly.V[ 1 ], edges[ es[ 1 ] ].Mid, edges[ es[ 0 ] ].Mid ) );
			mesh.Triangles().push_back( Triangle( poly.V[ 2 ], edges[ es[ 2 ] ].Mid, edges[ es[ 1 ] ].Mid ) );
			mesh.Triangles()[ i ] =     Triangle( edges[ es[ 0 ] ].Mid, edges[ es[ 1 ] ].Mid, edges[ es[ 2 ] ].Mid );
		}
		else if( count == 2 )
		{
			/*    .
			     / \
			    /___\
			   /     \
			  /_______\

			*/

			unsigned opp( 0 );
			for( unsigned j = 0; j < 3; ++j )
				if( !edges[ es[ j ] ].HasMid )
					opp = j;

			unsigned hit1( opp + 1 );
			if( hit1 > 2 )
				hit1 -= 3;

			unsigned hit2( hit1 + 1 );
			if( hit2 > 2 )
				hit2 -= 3;

			mesh.Quads().push_back( Quad( poly.V[ opp ], poly.V[ hit1 ], edges[ es[ hit1 ] ].Mid, edges[ es[ hit2 ] ].Mid ) );
			mesh.Triangles()[ i ] = Triangle( poly.V[ hit2 ], edges[ es[ hit2 ] ].Mid, edges[ es[ hit1 ] ].Mid );
		}
		else if( count == 1 )
		{	
			/*   .
			    /|\
			   / | \
			  /__|__\

			*/

			unsigned hit( 0 );
			for( unsigned j = 0; j < 3; ++j )
				if( edges[ es[ j ] ].HasMid )
					hit = j;

			unsigned opp( hit + 2 );
			if( opp > 2 )
				opp -= 3;

			unsigned n1( hit + 1 );
			if( n1 > 2 )
				n1 -= 3;

			mesh.Triangles().push_back( Triangle( edges[ es[ hit ] ].Mid, poly.V[ n1 ], poly.V[ opp ] ) );
			mesh.Triangles()[ i ] =     Triangle( edges[ es[ hit ] ].Mid, poly.V[ opp ], poly.V[ hit ] );
		}
	}

	for( unsigned i = 0; i < quad_edges.size(); ++i )
	{
		const Quad poly( mesh.Quads()[ i ] );
		const QuadEdges& pedges( quad_edges[ i ] );
		const unsigned count( SubCount( pedges, edges ) );
		/* There are four different quad subdivision cases, based on which
		   edges need to be cut. */
		if( count == 4 )
		{
			// Create center
			const unsigned cntr_ndx( mesh.VertexLocations().size() );
			mesh.VertexLocations().push_back( mesh.PolygonCenter( poly ) );

			mesh.Quads().push_back( Quad( cntr_ndx, edges[ pedges.E[ 3 ] ].Mid, poly.V[ 0 ], edges[ pedges.E[ 0 ] ].Mid ) );
			mesh.Quads().push_back( Quad( cntr_ndx, edges[ pedges.E[ 0 ] ].Mid, poly.V[ 1 ], edges[ pedges.E[ 1 ] ].Mid ) );
			mesh.Quads().push_back( Quad( cntr_ndx, edges[ pedges.E[ 1 ] ].Mid, poly.V[ 2 ], edges[ pedges.E[ 2 ] ].Mid ) );
			mesh.Quads()[ i ] =     Quad( cntr_ndx, edges[ pedges.E[ 2 ] ].Mid, poly.V[ 3 ], edges[ pedges.E[ 3 ] ].Mid );
		}
		else if( count == 3 )
		{
			unsigned unhit( 0 );
			// Find the side that isn't subd
			for( unsigned j = 0; j < 4; ++j )
				if( !edges[ pedges.E[ j ] ].HasMid )
					unhit = j;

			unsigned hit1( unhit + 1 );
			if( hit1 > 3 )
				hit1 -= 4;
			unsigned hit2( hit1 + 1 );
			if( hit2 > 3 )
				hit2 -= 4;
			unsigned hit3( hit2 + 1 );
			if( hit3 > 3 )
				hit3 -= 4;

			/*Triangles().push_back( Triangle( poly.V[ hit1 ], edges[ pedges.E[ hit2 ] ].Mid, poly.V[ unhit ] ) );
			Quads().push_back( Quad( poly.V[ hit1 ], edges[ pedges.E[ hit1 ] ].Mid, poly.V[ hit2 ], edges[ pedges.E[ hit2 ] ].Mid ) );
			Quads()[ i ] =             Quad( poly.V[ hit3 ], edges[ pedges.E[ hit3 ] ].Mid, poly.V[ unhit ], edges[ pedges.E[ hit2 ] ].Mid );*/
			mesh.Triangles().push_back( Triangle( edges[ pedges.E[ hit1 ] ].Mid, edges[ pedges.E[ hit2 ] ].Mid, edges[ pedges.E[ hit3 ] ].Mid ) );
			mesh.Triangles().push_back( Triangle( edges[ pedges.E[ hit1 ] ].Mid, poly.V[ hit2 ], edges[ pedges.E[ hit2 ] ].Mid ) );
			mesh.Triangles().push_back( Triangle( poly.V[ hit3 ], edges[ pedges.E[ hit3 ] ].Mid, edges[ pedges.E[ hit2 ] ].Mid ) );
			mesh.Quads()[ i ] = Quad( poly.V[ unhit ], poly.V[ hit1 ], edges[ pedges.E[ hit1 ] ].Mid, edges[ pedges.E[ hit3 ] ].Mid );
		}
		else if( count == 2 )
		{
			unsigned opp( 0 );
			for( unsigned j = 0; j < 4; ++j )
			{
				const unsigned j2( j == 3 ? 0 : j + 1 );
				if( !edges[ pedges.E[ j ] ].HasMid &&
				    !edges[ pedges.E[ j2 ] ].HasMid )
					opp = j2;
			}
			unsigned hit1( opp + 1 );
			if( hit1 > 3 )
				hit1 -= 4;
			unsigned hit2( opp + 2 );
			if( hit2 > 3 )
				hit2 -= 4;
			unsigned n1( opp + 1 );
			if( n1 > 3 )
				n1 -= 4;
			unsigned n2( hit2 + 1 );
			if( n2 > 3 )
				n2 -= 4;

			mesh.Triangles().push_back( Triangle( edges[ pedges.E[ hit2 ] ].Mid, poly.V[ n2 ], poly.V[ opp ] ) );
			mesh.Triangles().push_back( Triangle( poly.V[ n1 ], edges[ pedges.E[ hit1 ] ].Mid, poly.V[ opp ] ) );
			mesh.Quads()[ i ] = Quad( poly.V[ opp ], edges[ pedges.E[ hit1 ] ].Mid, poly.V[ hit2 ], edges[ pedges.E[ hit2 ] ].Mid );
		}
		else if( count == 1 )
		{
			// Find the side that is subd
			unsigned hit( 0 );
			for( unsigned j = 0; j < 4; ++j )
				if( edges[ pedges.E[ j ] ].HasMid )
					hit = j;

			// Find the point to which we will connect
			unsigned con( hit + 3 );
			if( con > 3 )
				con -= 4;

			mesh.Triangles().push_back( Triangle( poly.V[ hit ], edges[ pedges.E[ hit ] ].Mid, poly.V[ con ] ) );
			mesh.Quads()[ i ].V[ hit ] = edges[ pedges.E[ hit ] ].Mid;
		}
	}
}

void SubdivideBrush::Postwork( SculptStart )
{
}
void SubdivideBrush::Postwork( SculptContinue )
{
	Mesh& mesh( MeshHistory::Instance().GetCurrentMesh() );

	mesh.MakeSane();

	MeshHistory::Instance().SignalEntireSectionChanged().emit( true, true, true );
}
#endif
